unit MemoryMappedFiles;

{$mode objfpc}{$H+}

interface

uses
  Windows, Math, Classes, SysUtils, FileUtil, Forms, Controls, Graphics, Dialogs, StdCtrls;

const
MMF_MaxBytes = 4080;
SingleSize = sizeof(Single);
NAN_Zero = 0.0 + NaN;
NAN_Sleep = 1.0 + NaN;

type
  T_MMF_ByteArray= array[0..MMF_MaxBytes] of byte;
  T_MMF_ByteArrayPtr= ^T_MMF_ByteArray;

var
  MMF_Initialized: boolean = false;
  MMF_Filename: string;
  MMF_SharedByteArray: T_MMF_ByteArrayPtr;
  h_MMF_FileMapping: THandle;
  F_MMF_SharedByteArray: Pointer;

implementation

{$R *.lfm}

{ ================================== }
{       MMF - Private functions      }
{ ================================== }

procedure MMF_Initialize();
begin
  if h_MMF_FileMapping = 0 then
  begin
    F_MMF_SharedByteArray := nil;
    MMF_Filename := 'Theremino1';
    h_MMF_FileMapping:=CreateFileMapping(INVALID_HANDLE_VALUE, nil, PAGE_READWRITE, 0, sizeof(T_MMF_ByteArray), PChar(MMF_Filename));
    { Get a pointer to memory mapped file memory block }
    if h_MMF_FileMapping <> 0 then
    begin
      F_MMF_SharedByteArray := MapViewOfFile(h_MMF_FileMapping, FILE_MAP_ALL_ACCESS, 0, 0, 0);
      if Assigned(F_MMF_SharedByteArray) then MMF_Initialized := true;
    end
  end
end;

procedure MMF_Finalize();
begin
  CloseHandle(h_MMF_FileMapping);
end;

procedure MMF_WriteSingle(Offset: Integer; Value: Single);
var
  pSingle: ^Single;
begin
  if MMF_Initialized then
    if Offset >= 0 then
      if Offset < MMF_MaxBytes then
      begin
        pSingle := @Value;
        CopyMemory(F_MMF_SharedByteArray + Offset, pSingle, SingleSize);
      end
end;

function MMF_ReadSingle(Offset: Integer): Single;
var
  pSingle: ^Single;
  Value: Single;
begin
  if MMF_Initialized then
    if Offset >= 0 then
      if Offset < MMF_MaxBytes then
      begin
        pSingle := @Value;
        CopyMemory(pSingle, F_MMF_SharedByteArray + Offset, SingleSize);
      end;
  result := Value;
end;

{ ================================== }
{ Theremino SLOTS - Public functions }
{ ================================== }

procedure MemoryMappedFile_Init();
begin
  MMF_Initialize();
end;

function ReadSlot(Slot: Integer): Single;
var
  Value: Single;
begin
  Value := MMF_ReadSingle(Slot * SingleSize);
  result := Value;
end;

function ReadSlot_NoNan(Slot: Integer): Single;
var
  Value: Single;
begin
  Value := MMF_ReadSingle(Slot * SingleSize);
  if IsNaN(Value) then Value := 0.0;
  result := Value;
end;

procedure WriteSlot(Slot: Integer; Value: Single);
begin
  MMF_WriteSingle(Slot * SingleSize, Value);
end;

procedure MemoryMappedFile_FillWithNanSleep();
var
  Slot: Integer;
begin
  For Slot := 0 to Trunc(MMF_MaxBytes / SingleSize) - 1 do
    MMF_WriteSingle(Slot * SingleSize, NAN_Sleep);
end;


end. {unit}


